unit CommonTypes;
{ CommonTypes by Alessandro Cappellozza
  version 0.8 02/2002
  http://digilander.Libero.it/Kappe/audioobject
}

interface
 uses Windows, Dialogs, Forms, Controls, StdCtrls, Classes, ExtCtrls, SysUtils,
      ComCtrls, RndmList, dynamic_bass, JvPlaylist;

  Type
       TWaveData = array [ 0..2048] of DWORD;
       TFFTData  = array [0..1024] of Single;

       TsapMPDockType = (smdockFree, smdockLeft, smdockRight);
       TsongInfo = record
        Author : string;
        Title  : string;
        Date   : string;
        Stereo : Boolean;
        FileName : String;
       end;

      PPlayListNode = ^TPlayListNode;
      TPlayListNode = record
        SongInfo     : TSongInfo;
        LongFileName : String;
        ShortFileName: String;
      end;

      TPlayListMode = (plmNormal,plmRandom);

procedure ClearPlayListItems(pList : TListView);
procedure ClearPlayList(pList : TListView);
procedure AddPlayListItem(fn : String; pList : TListView);
function  GetPlayListitem(pList : TListView; First : Boolean = False) :String;
procedure SavePlayList(pList : TListView; FileName : String = 'sapMP.pla');
procedure LoadPlayList(FileName : String; pList : TListView);
function  LoadSongFromPlayList(pList : TJvPlaylist; var sn:String;Next:Boolean = True): Boolean;
function  ConvertTime(T:String):TTime;

{Main DSP}
procedure Rotate(handle: HDSP; channel: DWORD; buffer: Pointer; length, user: DWORD); stdcall;

  var
      d: ^DWORD; // DSP Handle
{Rotate DSP}
      rotpos: FLOAT;           // cur.pos

      SongInfo     : TSongInfo;
      PlayListNode : PPlayListNode;
      PlayingItem  : TListItem;
      PlayListMode : TPlayListMode;
      MusicLoaded  : Boolean;
      RandomList   : TRandomList;
      
implementation
uses StrUtils, unPlayList;

function  GetPlayListitem(pList : TListView; First : Boolean = False) :String;
var
 tl : TListItem;
begin
  Result := '';
  PlayingItem := nil;
  tl := pList.Selected;
  if First then tl := pList.Items[0];
  if tl = nil then Exit;
  if tl.Data<>nil then begin
   Result := PPlayListNode(tl.Data)^.LongFileName;
   PlayingItem := tl;
   if PlayListMode = plmRandom then RandomList.DeleteValue(tl.Index);
  end;
end;

procedure ClearPlayList(pList : TListView);
var i :integer;
   tl : TlistItem;
begin
 for i:=0 to pList.Items.Count-1 do
  if pList.Items[i].Data<>nil then
  begin
   PlayListNode := PPlayListNode(pList.Items[i].Data);
   Dispose(PlayListNode);
  end;
 RandomList.Clear; 
 pList.Clear;
 PlayingItem := nil;
end;

procedure ClearPlayListItems(pList : TListView);
var i : integer;
    tl,tl2 : TListItem;
begin
    tl := pList.GetNextItem(pList.Items[0],sdAll,[isSelected]);
    if tl = nil then Exit;
    repeat
//      tl.Selected then begin
        PlayListNode := PPlayListNode(tl.Data);
        Dispose(PlayListNode);
        tl2 := tl;
        tl := pList.GetNextItem(tl,sdAll,[isSelected]);
        if PlayListMode = plmRandom then RandomList.DeleteValue(tl2.Index);
        tl2.Delete;
    until tl=nil;
end;

procedure AddPlayListItem(fn : String; pList : TListView);
var i : integer;
    buf : array[0..1024] of char;
    tl : TListItem;
begin
 New(PlayListNode);
 PlayListNode^.LongFileName := fn;
 PlayListNode^.ShortFileName := ExtractShortPathName(fn);
 tl := pList.Items.Add;
 tl.Caption :=ExtractFileName(fn);
 tl.Data := PlayListNode;
// PlayingItem := tl;
 RandomList.InsertValue(tl.Index);
end;

procedure SavePlayList(pList : TListView; FileName : String = 'sapMP.pla');
var plFile : TextFile;
    i : integer;
    buf : array[0..1024] of char;
    relPath, shortName : String;
begin
 if Pos(':\',FileName)=0 then
  FileName := ExtractFileDir(Application.ExeName)+'\'+FileName;

 if FileExists(FileName) then DeleteFile(FileName);

 AssignFile(plFile,FileName);
 Rewrite(plFile);

 for i:=0 to pList.Items.Count-1 do
  if pList.Items[i].Data<>nil then
  begin
    PlayListNode := PPlayListNode(pList.Items[i].Data);
    relPath := ExtractRelativePath(FileName, PlayListNode^.LongFileName);
    shortName := ExtractRelativePath(ExtractShortPathName(FileName),PlayListNode^.ShortFileName);
// ----------------------------
    relPath := AnsiReplaceStr(relPath,'..\','\');
    relPath := AnsiReplaceStr(relPath,'..\','\');
    relPath := AnsiReplaceStr(relPath,'\\','\');

    shortName := AnsiReplaceStr(shortName,'..\','\');
    shortName := AnsiReplaceStr(shortName,'..\','\');
    shortName := AnsiReplaceStr(shortName,'\\','\');

    relPath := relPath+'|'+ShortName;
    WriteLn(plFile,relPath);
  end;
 CloseFile(plFile);
end;

procedure LoadPlayList(FileName : String; pList : TListView);
var plFile   : TextFile;
    StartDir : String;
    StDisk   : String;
    gl       : String;
begin
    StartDir := ExtractFileDir(FileName)+'\';
    stDisk := ExtractFileDrive(StartDir);
    AssignFile(plFile,FileName);
    Reset(plFile);

    while not Eof(plFile) do
    begin
     Readln(plFile,gl);
     Delete(gl,Pos('|',gl),Length(gl)-Pos('|',gl)+1);
     if gl[1] = '\' then gl := stDisk+gl else
        gl := StartDir+gl;

     AddPlayListItem(gl,pList);
    end;

    CloseFile(plFile);
end;

function LoadSongFromPlayList;
var
 tl : TListItem;
 ndx : integer;
begin
  Result :=  pList.Items.Count-1>0;
  if not Result then Exit;
  if sn <> '' then pList.ItemIndex := pList.Items.IndexOf(sn);
  if pList.ItemIndex = -1 then pList.ItemIndex := 0;
  if Next then sn := pList.Items[pList.ItemIndex+Integer((pList.ItemIndex<pList.Items.Count))]
  else sn := pList.Items[pList.ItemIndex-Integer((pList.ItemIndex>=1))];
  pList.ItemIndex := pList.Items.IndexOf(sn);
  Result := True;
end;

function  ConvertTime(T:String):TTime;
begin
 Result := EncodeTime(0,StrToInt(MidStr(T,1,Pos(':',T)-1)),StrToInt(MidStr(T,Pos(':',T)+1,2)),0);
end;

function fmod(a, b: FLOAT): FLOAT;
begin
  Result := a - (b * Trunc(a / b));
end;


procedure Rotate(handle: HDSP; channel: DWORD; buffer: Pointer; length, user: DWORD); stdcall;
var
  lc, rc: SmallInt;
begin
  d := buffer;
  while (length > 0) do
  begin
    lc := LOWORD(d^); rc := HIWORD(d^);
    lc := SmallInt(Trunc(sin(rotpos) * lc));
    rc := SmallInt(Trunc(cos(rotpos) * rc));
    d^ := MakeLong(lc, rc);
    Inc(d);
    rotpos := rotpos + fmod(0.00003, PI);
    length := length - 4;
  end;
end;


initialization
 RandomList := TRandomList.Create;
end.
