unit RndmList;

interface

uses
  Classes, SysUtils;

type
  EListEmpty = class(EListError)
    // Raised when list of values is empty
  end;

  TRandomList = class(TList)
  public
    constructor Create(Min, Max: Word);overload;
    constructor Create;overload;
    procedure Clear;override;
    destructor Destroy; override;
    function GetValue: Integer;
    function InsertValue(X: Word): Boolean;
    function DeleteValue(X: Word): Boolean;
    procedure Generate(Min,Max : Word);
  end;

implementation

type
  RandomNr = ^Word;

{-------------------- TRandomList ----------------------}

constructor TRandomList.Create(Min, Max: Word);
begin
  inherited Create;
  Generate(Min,Max);
end;

procedure TRandomList.Generate;
var
  P: RandomNr;
  I: Word;
begin
  for I := Min to Max do     // Insert records
  begin
    New(P);
    P^ := I;
    Add(P);
  end;
end;

constructor TRandomList.Create;
begin
  inherited Create;
end;

procedure TRandomList.Clear;
var
  I: Integer;
begin
  for I := 0 to Count -1 do
    Dispose(Items[I]);
  inherited Clear;
end;

destructor TRandomList.Destroy;
var
  I: Integer;
begin
  // We must clear the contents of the list
  for I := 0 to Count -1 do
    Dispose(Items[I]);
  // The pointers in the list will be destroyed automatically
  inherited Destroy;
end;


function TRandomList.GetValue: Integer;
var
  Selected: RandomNr;
begin
  if Count = 0 then Result := -1
  else
  begin
    Selected := Items[Random(Count)];
    Result := Selected^;
    Remove(Selected);
  end;
end;

function TRandomList.DeleteValue(X: Word): Boolean;
var
  I: Word;
  Found: Boolean;
begin
  I := 0;
  Found := False;
  while (not Found) and (I < Count) do
    if (Word(Items[I]^) = X) then
      Found := True
    else
      Inc(I);

  Result := Found;
  if Found then
    Delete(I);
end;

function TRandomList.InsertValue(X: Word): Boolean;
var
 P:RandomNr;
begin
   New(P);
   P^ := X;
   Add(P);
end;


end.

