unit spectrum_vis;
//
//  http://digilander.iol.it/Kappe/audioobject
//
interface
  uses Windows, Dialogs, Graphics, SysUtils, CommonTypes, Classes;

 type TSpectrum = Class(TObject)
    private
      VisBuff : TBitmap;
      FGRadient : TBitmap;
      BackBmp : TBitmap;

      BkgColor : TColor;
      SpecHeight : Integer;
      PenColor : TColor;
      PeakColor: TColor;
      DrawType : Integer;
      DrawRes  : Integer;
      FrmClear : Boolean;
      UseBkg   : Boolean;
      PeakFall : Integer;
      LineFall : Integer;
//      ColWidth : Integer;
      ShowPeak : Boolean;
      FYMax    : Integer;
      DrawText : String;

      FFTPeacks  : array [0..256] of Integer;
      FFTPeackT  : array [0..256] of Byte;
      FFTFallOff : array [0..256] of Integer;

    public
     ColWidth : Integer;
     Constructor Create (Width, Height : Integer);
     procedure Draw(HWND : THandle; FFTData : TFFTData; X, Y : Integer; FlushOnly : Boolean = True);
     procedure SetBackGround (Active : Boolean; BkgCanvas : TGraphic);

     property BackColor : TColor read BkgColor write BkgColor;
     property Height : Integer read SpecHeight write SpecHeight;
     property Width  : Integer read ColWidth write ColWidth;
     property Pen  : TColor read PenColor write PenColor;
     property Peak : TColor read PeakColor write PeakColor;
     property Mode : Integer read DrawType write DrawType;
     property Res  : Integer read DrawRes write DrawRes;
     property FrameClear : Boolean read FrmClear write FrmClear;
     property PeakFallOff: Integer read PeakFall write PeakFall;
     property LineFallOff: Integer read LineFall write LineFall;
     property DrawPeak   : Boolean read ShowPeak write ShowPeak;
  end;

 var Spectrum : TSpectrum;

implementation
uses unMain;

Constructor TSpectrum.Create(Width, Height : Integer);
var
 R, G, B, Y, X: Integer;
 C: TColor;
begin
  VisBuff := TBitmap.Create;
  BackBmp := TBitmap.Create;
  BackBmp.PixelFormat := pf32bit;
  VisBuff.PixelFormat := pf32bit;

    VisBuff.Width := Width;
    VisBuff.Height := Height;
    BackBmp.Width := Width;
    BackBmp.Height := Height;

    BkgColor := clBlack;
    SpecHeight := 100;
    DrawRes  := 2;
    FrmClear := True;
    UseBkg := False;
    PeakFall := 2;
    LineFall := 3;
    ColWidth := 4;
    ShowPeak := True;

  // Create gradient bitmap
  FGradient := TBitmap.Create;
  FGradient.PixelFormat := pf32bit;
  FGradient.Width := ColWidth;
  FGradient.Height := 22;
  FYMax    := FGradient.Height;

  R := 255;
  G := 0;
  B := 0;

  for Y := 0 to FGradient.Height-1 do
  begin
    if Y > 15 then
      Dec(R, 16)
    else
      Inc(G, 16);
    if R < 0 then
      R := 0;
    if G > 255 then
      G := 255;
    C := TColor(RGB(R, G, B));
    for X := 0 to ColWidth-2 do
      FGradient.Canvas.Pixels[X, Y] := C;
    FGradient.Canvas.Pixels[ColWidth-1, Y] := TColor(0);
  end;

end;

procedure TSpectrum.SetBackGround (Active : Boolean; BkgCanvas : TGraphic);
begin
  UseBkg := Active;
  BackBmp.Canvas.Draw(0, 0, BkgCanvas);
end;

procedure TSpectrum.Draw(HWND : THandle; FFTData : TFFTData; X, Y : Integer; FLushOnly : Boolean = True);
var i, YPos, yy, yy2 : LongInt;
    R, L : SmallInt;
    YVal, maxm : Single;
    x1,y1:LongInt;
begin

if FrmClear then begin
  VisBuff.Canvas.Pen.Color := BkgColor;
  VisBuff.Canvas.Brush.Color := BkgColor;
  VisBuff.Canvas.Rectangle(0, 0, VisBuff.Width, VisBuff.Height);
   if UseBkg then VisBuff.Canvas.CopyRect(Rect(0, 0, BackBmp.Width, BackBmp.Height), BackBmp.Canvas, Rect(0, 0, BackBmp.Width, BackBmp.Height));
end;

if not FlushOnly then
begin

VisBuff.Canvas.Pen.Color := PenColor;
 for i := 0 to ((VisBuff.Width-10) div ColWidth)-1 do begin
   YVal := Abs(FFTData[(i * DrawRes)+2]);
   YPos := Trunc((YVal) * 500);
   if YPos > FYMax  then YPos := FYMax;

   if YPos >= FFTPeacks[i] then begin
                                 FFTPeacks[i] := YPos;
                                 FFTPeackT[i] := 8;
                                end
      else
        if FFTPeackT[i] = 0 then FFTPeacks[i] :=  FFTPeacks[i] - PeakFall
                           else Dec(FFTPeackT[i],1);

      if YPos >= FFTFallOff[i] then FFTFallOff[i] := YPos
                               else FFTFallOff[i] := FFTFallOff[i] - LineFall;

      if (FGradient.Height - FFTPeacks[i]) > FYMax then FFTPeacks[i] := 0;
      if (FGradient.Height - FFTFallOff[i]) > FYMax then FFTFallOff[i] := 0;

       if ShowPeak then 
        if ColWidth = 1 then
        begin
          X1 := (i*ColWidth); if X1>((VisBuff.Width-10)-ColWidth) then X1:=((VisBuff.Width-10)-ColWidth);
          Y1 := (VisBuff.Height -4 - FFTPeacks[i]); if Y1>VisBuff.Height -4 then Y1:=VisBuff.Height -4;
          VisBuff.Canvas.Pixels[X+x1,Y1] := PeakColor;
        end
        else
        begin
          VisBuff.Canvas.Pen.Color := PeakColor;
          X1 := (i*ColWidth); if X1>((VisBuff.Width-10)-ColWidth) then X1:=((VisBuff.Width-10)-ColWidth);
          Y1 := (VisBuff.Height -4 - FFTPeacks[i]); if Y1>VisBuff.Height -4 then Y1:=VisBuff.Height -4;
          VisBuff.Canvas.MoveTo(X+x1,Y1);
          X1 := (i*ColWidth+ColWidth)-1; if X1>(VisBuff.Width-10) then X1:=(VisBuff.Width-10)-1;
          Y1 := (VisBuff.Height -4 - FFTPeacks[i]); if Y1>VisBuff.Height -4 then Y1:=VisBuff.Height -4;
          VisBuff.Canvas.LineTo(X+x1,Y1);
        end;

         yy :=(FGradient.Height - FFTFallOff[i]);
         yy2 := (VisBuff.Height -4 - FFTFallOff[i]);
         if yy<0 then yy :=0;

//        while yy<=FYMax-1 do
//        begin
          VisBuff.Canvas.CopyRect(Rect(X+(i*ColWidth), Yy2, X+(i*ColWidth+ColWidth), VisBuff.Height - 4), FGradient.Canvas, Rect(0, Yy, ColWidth, FGradient.Height));
//          inc(yy,1);
//        end; //while
  end;
 end; //if not Flush;
  BitBlt(HWND, 0, 0, VisBuff.Width, VisBuff.Height, VisBuff.Canvas.Handle, 0, 0, srccopy)
end;
end.

