{============================================================================
  BASS_FX.DLL v1.2 - Copyright (c) 2002-2003 JOBnik! [Arthur Aminov, ISRAEL]
  BASS_FX.PAS v1.2 - Copyright (c) 2002-2003 JOBnik! [Arthur Aminov, ISRAEL]

	 bugs/suggestions/questions -> e-mail: jobnik2k@hotmail.com
	------------------------------------------------------------

  BASS_FX 1.2 unit is based on BASS_FX 1.1 unit:
  ---------------------------------------------
  (c) 2002 Roger Johansson. w1dg3r@yahoo.com

  NOTE: This unit will only work with BASS_FX.DLL version 1.2
        Check http://www.un4seen.com/music for any later
        versions of BASS_FX.PAS

  * Requires BASS.DLL & BASS.PAS - available @ www.un4seen.com

  NOTE: TEMPO & BPM requires BASS.DLL v1.4 and greater.
        REVERSE     requires BASS.DLL v1.6 and greater.
	DSP         requires BASS.DLL v0.8 and greater.

  How to install:
  ---------------
  Copy BASS_FX.PAS & BASS.PAS to the \LIB subdirectory of your Delphi path
  or your project dir.
============================================================================}

unit BASS_FX;

interface

const
  // Use these to test for error from functions that return a DWORD or QWORD
  DW_ERROR = Cardinal(-1); // -1 (DWORD)
  QW_ERROR = Int64(-1);    // -1 (QWORD)

  // Error codes returned by BASS_FX_ErrorGetCode()
  BASS_FX_OK			= 0;	// All is OK
  BASS_FX_ERROR_MALLOC		= 1;	// Memory allocation error
  BASS_FX_ERROR_FREED		= 2;	// BASS_FX was freed
  BASS_FX_ERROR_BPMINUSE	= 3;	// BPM detection is in use
  BASS_FX_ERROR_BPMX2		= 4;	// BPM has been already mult by 2
  BASS_FX_ERROR_HANDLE		= 5;	// Invalid handle
  BASS_FX_ERROR_FREQ		= 6;	// Illegal sample rate
  BASS_FX_ERROR_NODECODE	= 7;	// Not a decoding channel
  BASS_FX_ERROR_FORMAT		= 8;	// File format is not supported
  BASS_FX_ERROR_STEREO		= 9;	// Only for stereo
  BASS_FX_ERROR_MODLEN		= 10;	// Music loaded without BASS_MUSIC_CALCLEN flag

  // BASS Error versions to use with BASS_FX
  BASS_FX_ERROR_BASS08		= 11;	// at least BASS 0.8 for DSP
  BASS_FX_ERROR_BASS14		= 12;	// at least BASS 1.4 for Tempo & BPM
  BASS_FX_ERROR_BASS16		= 13;	// at least BASS 1.6 for Reverse

type
  DWORD = cardinal;
  BOOL = LongBool;
  FLOAT = Single;
  QWORD = int64;           // 64-bit (replace "int64" with "comp" if using Delphi 3)

function BASS_FX_GetVersion: DWORD; stdcall; external 'bass_fx.dll' name 'BASS_FX_GetVersion';
{
	Retrieve the version number of BASS_FX that is loaded.
	RETURN : The BASS_FX version (LOWORD.HIWORD)
}

procedure BASS_FX_Free; stdcall; external 'bass_fx.dll' name 'BASS_FX_Free';
{
	Free all resources used BASS_FX, including: DSP, TEMPO, REVERSE & BPM.
	[Once this function called, you won't be able to set any FX!]
}

function BASS_FX_ErrorGetCode: DWORD; stdcall; external 'bass_fx.dll' name 'BASS_FX_ErrorGetCode';
{
	Get the BASS_FX_ERROR_xxx error code. Use this function to get the
	reason for an error.
}

{=============================================================================================
	D S P (Digital Signal Processing)
==============================================================================================}

// DSP effects
const
  BASS_FX_DSPFX_SWAP            = 0;	// Swap channels: left<=>right : for STEREO Only!
  BASS_FX_DSPFX_ROTATE          = 1;	// A channels volume ping-pong : for STEREO Only!
  BASS_FX_DSPFX_ECHO            = 2;    // Echo
  BASS_FX_DSPFX_FLANGER         = 3;    // Flanger
  BASS_FX_DSPFX_VOLUME          = 4;    // Volume Amplifier : L/R for STEREO, L for MONO
  BASS_FX_DSPFX_PEAKEQ          = 5;    // Peaking Equalizer
  BASS_FX_DSPFX_REVERB          = 6;    // Reverb
  BASS_FX_DSPFX_LPF             = 7;    // Low Pass Filter
  BASS_FX_DSPFX_CUT             = 8;    // Sound Cutter
  BASS_FX_DSPFX_FLANGER2        = 9;    // Flanger 2.0!

  // New DSP effects v1.2
  BASS_FX_DSPFX_S2M             = 10;	// Stereo 2 Mono : for STEREO Only!
  BASS_FX_DSPFX_DAMP            = 11;	// Dynamic Amplification
  BASS_FX_DSPFX_AUTOWAH         = 12;	// Auto Wah
  BASS_FX_DSPFX_ECHO2           = 13;	// Echo 2.0!
  BASS_FX_DSPFX_PHASER          = 14;	// Phaser
  BASS_FX_DSPFX_ECHO21          = 15;	// Echo 2.1!


type
// Flanger
  BASS_FX_DSPFLANGER = record
    fWetDry : FLOAT;			// [0.........1]
    fSpeed : FLOAT;			// [0.........n]
  end;

// Echo
  BASS_FX_DSPECHO = record
    fLevel : FLOAT;			// [0.........1]
    lDelay : Integer;			// [1200..30000]
  end;

// Reverb
  BASS_FX_DSPREVERB = record
    fbLevel : FLOAT;			// [0.........1]
    lbDelay : Integer;			// [1200..10000]
  end;

// Volume
  BASS_FX_DSPVOLUME = record		// L/R for STEREO, L for MONO for both channels
    fLeft : FLOAT;			// [0....1....n]
    fRight : FLOAT;			// [0....1....n]
  end;

// Peaking Equalizer
  BASS_FX_DSPPEAKEQ = record
    lBand : Integer;			// [0..n] more bands => more memory & cpu usage
    lFreq : DWORD;			// current sample rate of a stream/music
    fBandwidth : FLOAT;			// in octaves [0..4] - Q is not in use
    fQ : FLOAT;                       	// the EE kinda definition [0..1] - Bandwidth is not in use
    fCenter : FLOAT;			// eg. 1000Hz
    fGain : FLOAT;			// in dB. eg. [-10..0..+10]
  end;

// Low Pass Filter
  BASS_FX_DSPLPF = record
    lFreq : DWORD;			// current samplerate of stream/music
    fResonance : FLOAT;			// [2..........10]
    fCutOffFreq : FLOAT;		// [200Hz..5000Hz]
  end;

// Cutter
  BASS_FX_DSPCUT = record
    lCutsPerBeat : Integer;		// [0..n] number of cuts per beat
    fBPM : FLOAT;			// [1..n]
    lFreq : DWORD;			// current samplerate of a stream/music
  end;

// Flanger v2.0!
  BASS_FX_DSPFLANGER2 = record
    fDelay : FLOAT;		       	// [1.1..3.1] in ms.
    lFreq : DWORD;         		// current samplerate of a stream/music
    fBPM : FLOAT;  	  	    	// [1......n]
    fWetDry : FLOAT;       		// [0.....10]
  end;

// Dynamic Amplification
  BASS_FX_DSPDAMP = record
    lTarget : Integer;			// target level			e.g: [30000]
    lQuiet : Integer; 			// quiet level			e.g: [  800]
    fRate : FLOAT;			// amp adjustment rate		e.g: [ 0.02]
    fGain : FLOAT;			// amplification level
    lDelay : Integer;			// delay before increasing level
  end;

// Auto WAH
  BASS_FX_DSPAUTOWAH = record
    dDryMix : Double;			// dry (unaffected) signal mix         [-2.....2]
    dWetMix : Double;			// wet (affected) signal mix           [-2.....2]
    dFeedback : Double;			// feedback                            [-1.....1]
    dRate : Double;			// rate of sweep in cycles per second  [0<...<10]
    dRange : Double;			// sweep range in octaves              [0<...<10]
    dFreq : Double;			// base frequency of sweep Hz          [0<..1000]
  end;

// Echo 2.0!
  BASS_FX_DSPECHO2 = record
    dDryMix : Double;			// dry (unaffected) signal mix         [-2.....2]
    dWetMix : Double;			// wet (affected) signal mix           [-2.....2]
    dFeedback : Double;			// feedback                            [-1.....1]
    dDelay : Double;			// delay sec                           [0<.....6]
  end;

// Phaser
  BASS_FX_DSPPHASER = record
    dDryMix : Double;			// dry (unaffected) signal mix         [-2.....2]
    dWetMix : Double;			// wet (affected) signal mix           [-2.....2]
    dFeedback : Double;			// feedback                            [-1.....1]
    dRate : Double;			// rate of sweep in cycles per second  [0<...<10]
    dRange : Double;			// sweep range in octaves              [0<...<10]
    dFreq : Double;			// base frequency of sweep             [0<..1000]
  end;

// Echo 2.1!
  BASS_FX_DSPECHO21 = record
    dDryMix : Double;			// dry (unaffected) signal mix         [-2.....2]
    dWetMix : Double;			// wet (affected) signal mix           [-2.....2]
    dDelay : Double;			// delay sec                           [0<.....6]
  end;

function BASS_FX_DSP_Set(handle: DWORD; dsp_fx: Integer): BOOL; stdcall; external 'bass_fx.dll' name 'BASS_FX_DSP_Set';
{
	Set any chosen DSP FX to any Handle.
	handle : stream/music/wma/cd
	dsp_fx : FX you wish to use
	RETURN : TRUE if created (0=error!)
}

function BASS_FX_DSP_Remove(handle: DWORD; dsp_fx: Integer): BOOL; stdcall; external 'bass_fx.dll' name 'BASS_FX_DSP_Remove';
{
	Remove chosen DSP FX.
	handle : stream/music/wma/cd
	dsp_fx : FX you wish to remove
	RETURN : TRUE if removed (0=error!)
}

function BASS_FX_DSP_SetParameters(handle: DWORD; dsp_fx: Integer; par: Pointer): BOOL; stdcall; external 'bass_fx.dll' name 'BASS_FX_DSP_SetParameters';
{
	Set the parameters of a DSP effect.
	handle : stream/music/wma/cd
	dsp_fx : FX you wish to set parameters to
	par    : Pointer to the parameter type
}

function BASS_FX_DSP_GetParameters(handle: DWORD; DSP_FX: Integer; par: Pointer): BOOL; stdcall; external 'bass_fx.dll' name 'BASS_FX_DSP_GetParameters';
{
	Set the parameters of a DSP effect.
	handle : stream/music/wma/cd
	dsp_fx : FX you wish to get parameters to
	par    : Pointer to the parameter type
}

function BASS_FX_DSP_Reset(handle: DWORD; dsp_fx: Integer): BOOL; stdcall; external 'bass_fx.dll' name 'BASS_FX_DSP_Reset';
{
	Call this function before changing position to avoid *clicks*
	handle : stream/music/wma/cd
	dsp_fx : FX you wish to reset parameters of
	RETURN : TRUE if ok (0=error!)
}

{=============================================================================================
	TEMPO / PITCH SCALING / SAMPLERATE
==============================================================================================}

// tempo flags
const BASS_FX_TEMPO_QUICKSEEK = 512;     // If you use quicker tempo change algorithm (gain speed, lose quality)

function BASS_FX_TempoCreate(chan, flags: DWORD): DWORD; stdcall; external 'bass_fx.dll' name 'BASS_FX_TempoCreate';
{
	Creates a Resampling stream from a decoding channel.
	chan   : a Handle returned by:
        		BASS_StreamCreateFile     : flags = BASS_STREAM_DECODE ...
			BASS_MusicLoad            : flags = BASS_MUSIC_DECODE ...
			BASS_WMA_StreamCreateFile : flags = BASS_STREAM_DECODE ...
	        	BASS_CD_StreamCreate      : flags = BASS_STREAM_DECODE ...
	flags  : BASS_SAMPLE_xxx / BASS_SPEAKER_xxx / BASS_FX_TEMPO_QUICKSEEK
	RETURN : the resampled handle (0=error!)
}

function BASS_FX_TempoSet(chan: DWORD; fTempo: FLOAT; lSampleRate: Integer; fPitchScale: FLOAT): BOOL; stdcall; external 'bass_fx.dll' name 'BASS_FX_TempoSet';
{
	Set new values to tempo/rate/pitch to change it's speed.
	chan        : stream/music/wma/cd
	fTempo      : in Percents  [-95%..0..+5000%]   			(-100.0 if not in use)
	lSampleRate : in Hz, but calculates by the same % as fTempo	(-1     if not in use)
	fPitchScale : in Semitones [-60....0....+60]   			(-100.0 if not in use)
	RETURN      : TRUE if ok (0=error!)
}

function BASS_FX_TempoGet(chan: DWORD; var fTempo: FLOAT; var lSampleRate: DWORD; var fPitchScale: FLOAT): BOOL; stdcall; external 'bass_fx.dll' name 'BASS_FX_TempoGet';
{
	Get tempo/rate/pitch values.
        chan        : stream/music/wma/cd
        fTempo      : current tempo         (NULL if not in use)
        lSampleRate : current samplerate    (NULL if not in use)
        fPitchScale : current pitch         (NULL if not in use)
	RETURN      : TRUE if ok (0=error!)
}

function BASS_FX_TempoGetApproxSeconds(chan: DWORD; sec: FLOAT): FLOAT; stdcall; external 'bass_fx.dll' name 'BASS_FX_TempoGetApproxSeconds';
{
	Get the approximate seconds from given seconds after changing Tempo or Rate.
        chan   : stream/music/wma/cd
        sec    : seconds to convert
	RETURN : new calculated length in seconds (-1=error!)
}

function BASS_FX_TempoGetApproxPercents(chan: DWORD): FLOAT; stdcall; external 'bass_fx.dll' name 'BASS_FX_TempoGetApproxPercents';
{
	Get the approximate percents of changed Tempo/Rate after changing Tempo or Rate.
        chan   : stream/music/wma/cd
	RETURN : the percents (-1=error!)
}

function BASS_FX_TempoGetResampledHandle(chan: DWORD): DWORD; stdcall; external 'bass_fx.dll' name 'BASS_FX_TempoGetResampledHandle';
{
	Get the resampled handle from a decoding handle.
        chan   : stream/music/wma/cd
	RETURN : the resampled handle (0=error!)
}

function BASS_FX_TempoStopAndFlush(chan: DWORD): BOOL; stdcall; external 'bass_fx.dll' name 'BASS_FX_TempoStopAndFlush';
{
	Stop a stream and clear it's buffer.
        chan   : stream/music/wma/cd
	RETURN : TRUE if ok (0=error!)
}

procedure BASS_FX_TempoFree(chan: DWORD); stdcall; external 'bass_fx.dll' name 'BASS_FX_TempoFree';
{
	Free all the resources used by a given handle.
        chan : stream/music/wma/cd
}

{=============================================================================================
	R E V E R S E
==============================================================================================}

{
        NOTE: Reverse will NOT work with MOD/WMA/CD formats!
              * Will work with WMA & CD, only with BASS version 1.8 and greater!
	        if so, then requires much larger dec_block ;)
}

function BASS_FX_ReverseCreate(chan: DWORD; dec_block: FLOAT; decode: BOOL; flags: DWORD): DWORD; stdcall; external 'bass_fx.dll' name 'BASS_FX_ReverseCreate';
{
	Creates a Reversed stream from a decoding channel.
	chan      : a Handle returned by:
                       BASS_StreamCreateFile     : flags = BASS_STREAM_DECODE
                       BASS_WMA_StreamCreateFile : flags = BASS_STREAM_DECODE ...  (BASS 1.8+)
                       BASS_CD_StreamCreate      : flags = BASS_STREAM_DECODE ...  (BASS 1.8+)
		       * For better MP3/2/1 Reverse playback use: BASS_MP3_SETPOS flag
	dec_block : decode in # of seconds blocks...
				larger blocks = less seeking overhead but larger spikes.
	decode	  : allow (TRUE/FALSE) the reversed stream to be created as a decoding channel.
	flags	  : BASS_SAMPLE_xxx / BASS_SPEAKER_xxx
	RETURN    : The reversed handle (0=error!)
}

function BASS_FX_ReverseGetReversedHandle(chan: DWORD): DWORD; stdcall; external 'bass_fx.dll' name 'BASS_FX_ReverseGetReversedHandle';
{
	Get the reversed handle from a decoding channel.
	chan   : stream handle
	RETURN : The reversed handle (0=error!)
}

function BASS_FX_ReverseSetPosition(chan: DWORD; pos: QWORD): BOOL; stdcall; external 'bass_fx.dll' name 'BASS_FX_ReverseSetPosition';
{
	Change the playing position of a decoding channel.
	chan   : stream handle
	pos    : the position (in bytes)
	RETURN : TRUE if ok (0=error!)
}

procedure BASS_FX_ReverseFree(chan: DWORD); stdcall; external 'bass_fx.dll' name 'BASS_FX_ReverseFree';
{
	Free all resources used by a given handle.
	chan : stream handle
}


{=============================================================================================
	B P M (Beats Per Minute)
=============================================================================================}

// bpm flags
const
  BASS_FX_BPM_BKGRND = 1;   // If in use, then you can do other processing while detecting
  BASS_FX_BPM_MULT2 = 2;    // If in use, then will auto multiple bpm by 2 (if BPM < MinBPM*2)

//-----------
// Option - 1 - Get BPM from a Decoded Channel
//-----------

type BPMPROCESSPROC = procedure(chan: DWORD; percent: FLOAT); stdcall;
{
	Get the detection process in percents of a channel.
	chan    : channel that the BASS_FX_BPM_GetDecode is being applied to
	percent : the process in percents [0%..100%]
}

function BASS_FX_BPM_DecodeGet(chan: DWORD; fStartSec, fEndSec: FLOAT; lMinMaxBPM, flags: DWORD; proc: BPMPROCESSPROC): FLOAT; stdcall; external 'bass_fx.dll' name 'BASS_FX_BPM_DecodeGet';
{
	Get the original BPM of a decoding channel with original frequency
	chan      : a Handle returned by:
			BASS_StreamCreateFile     : flags = BASS_STREAM_DECODE ...
			BASS_MusicLoad            : flags = BASS_MUSIC_DECODE Or BASS_MUSIC_CALCLEN ...
   		        BASS_WMA_StreamCreateFile : flags = BASS_STREAM_DECODE ...  (BASS 1.8+)
			BASS_CD_StreamCreate      : flags = BASS_STREAM_DECODE ...  (BASS 1.8+)
	fStartSec : start detecting position in seconds
	fEndSec   : end detecting position in seconds - 0 = full length
	lMinMaxBPM: set min & max bpm, e.g: MAKELONG(LOWORD.HIWORD), LO=Min, HI=Max. 0 = defaults 45/230
	flags	  : BASS_FX_BPM_xxx
	proc      : user defined function to receive the process in percents, use NULL if not in use
	RETURN    : the original BPM value (-1=error!)
}

//-----------
// Option - 2 - Auto Get BPM by Period of time in seconds.
//-----------

type BPMPROC = procedure(handle: DWORD; bpm: FLOAT); stdcall;
{
	Get the BPM after period of time in seconds.
	handle : handle that the BASS_FX_BPM_CallbackSet is being applied to
	bpm    : the new original bpm value
}

function BASS_FX_BPM_CallbackSet(handle: DWORD; proc: BPMPROC; fPeriod: FLOAT; lMinMaxBPM, flags: DWORD): BOOL; stdcall; external 'bass_fx.dll' name 'BASS_FX_BPM_CallbackSet';
{
	Enable getting BPM value by period of time in seconds.
	handle    : stream/music/wma/cd
	proc      : user defined function to receive the bpm value
	fPeriod	  : detection period in seconds
	lMinMaxBPM: set min & max bpm, e.g: MAKELONG(LOWORD.HIWORD), LO=Min, HI=Max. 0 = defaults 45/230
	flags     : only BASS_FX_BPM_MULT2 flag is used
}

function BASS_FX_BPM_CallbackReset(handle: DWORD): BOOL; stdcall; external 'bass_fx.dll' name 'BASS_FX_BPM_CallbackReset';
{
	Reset the buffers. Call this function after changing position.
	handle : stream/music/wma/cd
}

{-------------------------------------------------------
   Functions to use with Both options.
	NOTE: These functions will not detect the BPM,
              they will just calculate the detected
              original BPM value of a given handle.
-------------------------------------------------------}

function BASS_FX_BPM_X2(handle: DWORD): FLOAT; stdcall; external 'bass_fx.dll' name 'BASS_FX_BPM_X2';
{
	Multiple the original BPM value by 2 (may be called only once).
	handle : stream/music/wma/cd/decoding channel
	RETURN : New BPM value * 2 (-1=error!)
}

function BASS_FX_BPM_Frequency2BPM(handle: DWORD; freq: DWORD): FLOAT; stdcall; external 'bass_fx.dll' name 'BASS_FX_BPM_Frequency2BPM';
{
	Convert Frequency to BPM value.
	handle : stream/music/wma/cd/decoding channel
	freq   : frequency (e.g 44100).
	RETURN : New BPM value for that handle (-1=error!)
}

function BASS_FX_BPM_2Frequency(handle: DWORD; bpm: FLOAT): DWORD; stdcall; external 'bass_fx.dll' name 'BASS_FX_BPM_2Frequency';
{
	Convert BPM value to Frequency.
	handle : stream/music/wma/cd/decoding channel
	bpm    : a bpm value (e.g 140.32).
	RETURN : New Frequency for that handle (-1=error!)
}

function BASS_FX_BPM_Percents2BPM(handle: DWORD; per: FLOAT): FLOAT; stdcall; external 'bass_fx.dll' name 'BASS_FX_BPM_Percents2BPM';
{
	Convert Percents to BPM value.
	handle : stream/music/wma/cd/decoding channel
	per    : percent
	RETURN : New BPM value for that handle (-1=error!)
}

function BASS_FX_BPM_2Percents(handle: DWORD; bpm: FLOAT): FLOAT; stdcall; external 'bass_fx.dll' name 'BASS_FX_BPM_2Percents';
{
	Convert BPM to Percents.
	handle : stream/music/wma/cd/decoding channel
	bpm    : a bpm value
	RETURN : New Percents for that handle (-1=error!)
}

procedure BASS_FX_BPM_Free(chan: DWORD); stdcall; external 'bass_fx.dll' name 'BASS_FX_BPM_Free';
{
	Free all resources used by a given handle.
	chan : stream/music/wma/cd/decoding channel
}

implementation

end.